@@ -25,6 +25,7 @@
 #' @return A valid \code{SoilProfileCollection} object.
 #' @author D.E. Beaudette, A.G. Brown
 #' @seealso \code{\link{checkSPC}}
+#' @export
 rebuildSPC <- function(x) {
 
   # break into pieces as list
@@ -40,59 +41,71 @@ rebuildSPC <- function(x) {
 
   # preserve original hzidname
   # this is missing in old SPC objects
-  if(!is.null(x.list$hzidcol)) {
+  if (!is.null(x.list$hzidcol)) {
     hzidname(res) <- x.list$hzidcol
   }
-
-  if(is.null(x.list$hzdesgncol)) {
+  
+  if (is.null(x.list$hzdesgncol)) {
     x.list$hzdesgncol <- ""
   }
-
+  
   # preserve original hzdesgnname
   # this is missing in old SPC objects
-  if(!is.null(x.list$hzdesgncol)) {
+  if (!is.null(x.list$hzdesgncol)) {
     hzdesgnname(res) <- x.list$hzdesgncol
   }
-
-  if(is.null(x.list$hztexclcol)) {
+  
+  if (is.null(x.list$hztexclcol)) {
     x.list$hztexclcol <- ""
   }
-
+  
   # preserve original hztexclname
   # this is missing in old SPC objects
-  if(!is.null(x.list$hztexclcol)) {
+  if (!is.null(x.list$hztexclcol)) {
     hztexclname(res) <- x.list$hztexclcol
   }
-
+  
   # transfer metadata
   res <- .transfer.metadata.aqp(x, res)
-    
+  
   # replace site
   site(res) <- x.list$site
-
+  
   # @diagnostic and may be missing if `x` is very old
   # replace with empty data.frame so that setters do not error
-  if(is.null(x.list$diagnostic)) {
+  if (is.null(x.list$diagnostic)) {
     x.list$diagnostic <- data.frame()
   }
-
+  
   # @restructions may be missing if `x` is very old
   # replace with empty data.frame so that setters do not error
-  if(is.null(x.list$restrictions)) {
+  if (is.null(x.list$restrictions)) {
     x.list$restrictions <- data.frame()
   }
-
+  
   # set
   diagnostic_hz(res) <- x.list$diagnostic
   restrictions(res) <- x.list$restrictions
 
-  # copy valid spatial data from the source object
-  # otherwise, the previous and possibly invalid SpatialPoints object created by horizons() is fine
-  if(validSpatialData(x)) {
-    res@sp <- x.list$sp
+  # 2022-12-21: aqp 2.0; sp slot is no longer transferred by rebuildSPC
+  #             but we transfer any existing data back to site 
+  if (!is.null(x.list$sp)) {
+    if (inherits(x.list$sp, 'SpatialPoints') &&
+        ncol(x.list$sp@coords) == 2) {
+      newsp <- cbind(data.frame(id = profile_id(x), stringsAsFactors = FALSE), 
+                     as.data.frame(x.list$sp, stringsAsFactors = FALSE))
+      
+      crdnms <- c("x", "y")
+      if (any(crdnms %in% names(x))) {
+        crdnms <-  paste0("sp.", crdnms)
+      } 
+      colnames(newsp) <- c(idname(x), crdnms)
+      
+      site(res) <- newsp
+      initSpatial(res) <- colnames(newsp)[2:3]
+      try(prj(res) <- x.list$sp@proj4string)
+    }
   }
-
-  # done
   return(res)
 }
 
