@@ -76,6 +76,7 @@
 #' 
 #' @rdname texture
 #' @keywords manip
+#' @export
 #' @examples
 #' \donttest{
 #' # example of ssc_to_texcl()
@@ -260,12 +261,12 @@ texcl_to_ssc <- function(texcl = NULL, clay = NULL, sample = FALSE) {
 }
 
 #' Convert sand, silt and clay to texture class
-#'
+#' @param simplify Passed to `SoilTextureLevels()` to set the nu mber of possible texture classes. If `TRUE`, the ordered factor has a maximum of 12 levels, if `FALSE` (default) the ordered factor has a maximum of 21 levels (including e.g. very fine/fine/coarse variants)
 #' @rdname texture
 #' @return  - `ssc_to_texcl`: A `character` vector containing texture class
 #' @export
 #'
-ssc_to_texcl <- function(sand = NULL, clay = NULL, as.is = FALSE, droplevels = TRUE) {
+ssc_to_texcl <- function(sand = NULL, clay = NULL, simplify = FALSE, as.is = FALSE, droplevels = TRUE) {
   # fix for R CMD check:
   #  ssc_to_texcl: no visible binding for global variable ‘silt’
   silt <- NULL
@@ -312,10 +313,10 @@ ssc_to_texcl <- function(sand = NULL, clay = NULL, as.is = FALSE, droplevels = T
   })
 
   # encoding according to approximate AWC, from Red Book version 3.0
-  if (as.is == FALSE) {
-    df$texcl <- factor(df$texcl, levels = SoilTextureLevels(which = 'codes'), ordered = TRUE)
+  if (!as.is) {
+    df$texcl <- factor(df$texcl, levels = SoilTextureLevels(which = 'codes', simplify = simplify), ordered = TRUE)
 
-    if (droplevels == TRUE) {
+    if (droplevels) {
       df$texcl <- droplevels(df$texcl)
     }
   }
